@@ -12,15 +12,14 @@
 -- A set of /hashable/ values.  A set cannot contain duplicate items.
 -- A 'HashSet' makes no guarantees as to the order of its elements.
 --
--- The implementation is based on /big-endian patricia trees/, indexed
--- by a hash of the original value.  A 'HashSet' is often faster than
--- other tree-based set types, especially when value comparison is
--- expensive, as in the case of strings.
+-- The implementation is based on /hash array mapped trie/.  A
+-- 'HashSet' is often faster than other tree-based set types,
+-- especially when value comparison is expensive, as in the case of
+-- strings.
 --
--- Many operations have a worst-case complexity of /O(min(n,W))/.
--- This means that the operation can become linear in the number of
--- elements with a maximum of /W/ -- the number of bits in an 'Int'
--- (32 or 64).
+-- Many operations have a average-case complexity of /O(log n)/.  The
+-- implementation uses a large base (i.e. 16) so in practice these
+-- operations are constant time.
 
 module Data.HashSet
     (
@@ -32,6 +31,7 @@ module Data.HashSet
 
     -- * Combine
     , union
+    , unions
 
     -- * Basic interface
     , null
@@ -60,7 +60,7 @@ module Data.HashSet
     ) where
 
 import Control.DeepSeq (NFData(..))
-import Data.HashMap.Common (HashMap, foldrWithKey)
+import Data.HashMap.Base (HashMap, foldrWithKey)
 import Data.Hashable (Hashable)
 import Data.Monoid (Monoid(..))
 import Prelude hiding (filter, foldr, map, null)
@@ -112,7 +112,7 @@ singleton a = HashSet (H.singleton a ())
 {-# INLINABLE singleton #-}
 #endif
 
--- | /O(n)/ Construct a set containing all elements from both sets.
+-- | /O(n+m)/ Construct a set containing all elements from both sets.
 --
 -- To obtain good performance, the smaller set must be presented as
 -- the first argument.
@@ -120,6 +120,13 @@ union :: (Eq a, Hashable a) => HashSet a -> HashSet a -> HashSet a
 union s1 s2 = HashSet $ H.union (asMap s1) (asMap s2)
 {-# INLINE union #-}
 
+-- TODO: Figure out the time complexity of 'unions'.
+
+-- | Construct a set containing all elements from a list of sets.
+unions :: (Eq a, Hashable a) => [HashSet a] -> HashSet a
+unions = List.foldl' union empty
+{-# INLINE unions #-}
+
 -- | /O(1)/ Return 'True' if this set is empty, 'False' otherwise.
 null :: HashSet a -> Bool
 null = H.null . asMap
